@@ -9,6 +9,7 @@
 #' @param feature.mode the mode of how to define the features thereof. It can be: "supra" for combinations of one or two successive domains (including individual domains; considering the order), "individual" for individual domains only, and "comb" for all possible combinations (including individual domains; ignoring the order)
 #' @param min.overlap the minimum number of overlaps with each term in consideration. By default, it sets to a minimum of 3
 #' @param fdr.cutoff the fdr cutoff to call the significant associations between features and terms. By default, it sets to 1e-3
+#' @param hscore.type the type of defining the hypergeometric score. It can be: "zscore" for z-score (by default), "fdr" for fdr (after being transformed via \eqn{-1*log_2(fdr)})
 #' @param parallel logical to indicate whether parallel computation with multicores is used. By default, it sets to true, but not necessarily does so. Partly because parallel backends available will be system-specific (now only Linux or Mac OS). Also, it will depend on whether these two packages "foreach" and "doMC" have been installed. It can be installed via: \code{source("http://bioconductor.org/biocLite.R"); biocLite(c("foreach","doMC"))}. If not yet installed, this option will be disabled
 #' @param multicores an integer to specify how many cores will be registered as the multicore parallel backend to the 'foreach' package. If NULL, it will use a half of cores available in a user's computer. This option only works when parallel computation is enabled
 #' @param verbose logical to indicate whether the messages will be displayed in the screen. By default, it sets to TRUE for display
@@ -28,11 +29,11 @@
 #' anno.file <- "http://dcgor.r-forge.r-project.org/data/Algo/HP_anno.txt"
 #' architecture.file <- "http://dcgor.r-forge.r-project.org/data/Algo/SCOP_architecture.txt"
 #'
-#' # 2) Do inference using built-in data
+#' # 2) Do inference using built-in ontology
 #' res <- dcAlgo(anno.file, architecture.file, ontology="HPPA", feature.mode="supra", parallel=FALSE)
 #' res[1:5,]
 #' 
-#' # 3) Advanced usage: using customised data
+#' # 3) Advanced usage: using customised ontology
 #' x <- base::load(base::url("http://dcgor.r-forge.r-project.org/data/onto.HPPA.RData"))
 #' RData.ontology.customised <- 'onto.HPPA.RData'
 #' base::save(list=x, file=RData.ontology.customised)
@@ -42,7 +43,7 @@
 #' res[1:5,]
 #' }
 
-dcAlgo <- function(anno.file, architecture.file, output.file=NULL, ontology=c(NA,"GOBP","GOMF","GOCC","DO","HPPA","HPMI","HPON","MP","EC","KW","UP"), feature.mode=c("supra","individual","comb"), min.overlap=3, fdr.cutoff=1e-3, parallel=TRUE, multicores=NULL, verbose=T, RData.ontology.customised=NULL, RData.location="http://dcgor.r-forge.r-project.org/data")
+dcAlgo <- function(anno.file, architecture.file, output.file=NULL, ontology=c(NA,"GOBP","GOMF","GOCC","DO","HPPA","HPMI","HPON","MP","EC","KW","UP"), feature.mode=c("supra","individual","comb"), min.overlap=3, fdr.cutoff=1e-3, hscore.type=c("zscore","fdr"), parallel=TRUE, multicores=NULL, verbose=T, RData.ontology.customised=NULL, RData.location="http://dcgor.r-forge.r-project.org/data")
 {
 
     startT <- Sys.time()
@@ -53,6 +54,7 @@ dcAlgo <- function(anno.file, architecture.file, output.file=NULL, ontology=c(NA
     ## match.arg matches arg against a table of candidate values as specified by choices, where NULL means to take the first one
     ontology <- match.arg(ontology)
     feature.mode <- match.arg(feature.mode)
+    hscore.type <- match.arg(hscore.type)
     
     p.adjust.method <- c("BH","BY","bonferroni","holm","hochberg","hommel")[1]
     min.overlap <- as.integer(min.overlap)
@@ -170,15 +172,34 @@ dcAlgo <- function(anno.file, architecture.file, output.file=NULL, ontology=c(NA
         #########
         ## get final hscore and fdr
         hscore <- signif(apply(cbind(zscore,zscore.rel), 1, min), digits=2)
-        fdr <- signif(apply(cbind(adjpvals,adjpvals.rel), 1, max), digits=2)
-    
-        res <- data.frame(Feature_id=rep(name.group,length(X)), Term_id=names(M), X, M, K=rep(K,length(X)), N=rep(N,length(X)), zscore, p.value, adjpvals, K_rel, N_rel, zscore.rel, p.value.rel, adjpvals.rel, Score=hscore, fdr, row.names=NULL)
+        #pval <- apply(cbind(p.value,p.value.rel), 1, max)
+        #fdr <- stats::p.adjust(pval, method=p.adjust.method)
+        fdr <- apply(cbind(adjpvals,adjpvals.rel), 1, max)
+
+        flag <- fdr==0
+        fdr[flag] <- min(fdr[!flag])
         
-        ind <- which(res$X >= min.overlap & res$fdr < fdr.cutoff)
-        if(length(ind)>=1){
-            res[ind,c(1:2,15)]
-        }else{
-            NULL   
+        res <- data.frame(Feature_id=rep(name.group,length(X)), Term_id=names(M), X, M, K=rep(K,length(X)), N=rep(N,length(X)), zscore, p.value, adjpvals, K_rel, N_rel, zscore.rel, p.value.rel, adjpvals.rel, hscore, fdr, row.names=NULL)
+        
+        if(hscore.type=='zscore'){
+
+            ind <- which(res$X >= min.overlap & res$fdr < fdr.cutoff)
+            if(length(ind)>=1){
+                return(res[ind,c(1:2,15)])
+            }else{
+                return(NULL)
+            }
+            
+        }else if(hscore.type=='fdr'){
+            
+            ind <- which(res$X >= min.overlap & res$fdr < fdr.cutoff)
+            res$fdr <- signif(-1*log10(res$fdr), digits=2)
+            if(length(ind)>=1){
+                return(res[ind,c(1:2,16)])
+            }else{
+                return(NULL)
+            }
+            
         }
 
     }
@@ -190,16 +211,18 @@ dcAlgo <- function(anno.file, architecture.file, output.file=NULL, ontology=c(NA
     }
     
     ## import annotations
-    tab <- read.delim(anno.file, header=F, sep="\t", nrows=50, skip=1)
-    anno <- read.table(anno.file, header=F, sep="\t", skip=1, colClasses=sapply(tab,class))
+    #tab <- utils::read.delim(anno.file, header=F, sep="\t", nrows=50, skip=1)
+    #anno <- utils::read.table(anno.file, header=F, sep="\t", skip=1, colClasses=sapply(tab,class))
+    anno <- utils::read.delim(anno.file, header=T, sep="\t", colClasses="character")
     ## import architectures
-    tab <- read.delim(architecture.file, header=F, sep="\t", nrows=50, skip=1)
-    arch <- read.table(architecture.file, header=F, sep="\t", skip=1, colClasses=sapply(tab,class))
+    #tab <- utils::read.delim(architecture.file, header=F, sep="\t", nrows=50, skip=1)
+    #arch <- utils::read.table(architecture.file, header=F, sep="\t", skip=1, colClasses=sapply(tab,class))
+    arch <- utils::read.delim(architecture.file, header=T, sep="\t", colClasses="character")
     
     ## replace proteins with internal id
-    all <- unique(c(unique(as.character(anno[,1])), unique(as.character(arch[,1]))))
-    anno[,1] <- match(as.character(anno[,1]), all)
-    arch[,1] <- match(as.character(arch[,1]), all)
+    all <- unique(c(unique(anno[,1]), unique(arch[,1])))
+    anno[,1] <- match(anno[,1], all)
+    arch[,1] <- match(arch[,1], all)
     
     ###############################################################################################
     if(verbose){
@@ -427,11 +450,18 @@ dcAlgo <- function(anno.file, architecture.file, output.file=NULL, ontology=c(NA
     
     feature2term_score <- base::do.call(base::rbind, res_list)
     rownames(feature2term_score) <- NULL
+    colnames(feature2term_score) <- c("Feature_id", "Term_id", "Score")
+    
+    # how to deal with non-positive scores
+    if(1){
+        ## only retain all entries with positive score (otherwise, there will be problematic for prediction)
+        feature2term_score <- feature2term_score[as.numeric(feature2term_score[,3])>0, ]
+    }
     
     if(!is.null(output.file)){
     
         output <- feature2term_score
-        write.table(output, file=output.file, quote=F, row.names=F, sep="\t")
+        utils::write.table(output, file=output.file, quote=F, row.names=F, sep="\t")
         
         if(file.exists(output.file)){
             message(sprintf("The results have been saved into '%s'.", file.path(getwd(),output.file)), appendLF=T)
