@@ -3,6 +3,7 @@
 #' \code{dcFunArgs} is supposed to assign (and evaluate) arguments with default values for an input function.
 #'
 #' @param fun an input function name (character string)
+#' @param action logical to indicate whether the function will act as it should be (with assigned values in the current environment). By default, it sets to FALSE
 #' @param verbose logical to indicate whether the messages will be displayed in the screen. By default, it sets to TRUE for display
 #' @return
 #' a list containing arguments and their default values
@@ -15,7 +16,7 @@
 #' fun <- "dcAlgoPredictMain"
 #' dcFunArgs(fun)
 
-dcFunArgs <- function(fun, verbose=T)
+dcFunArgs <- function(fun, action=F, verbose=T)
 {
     
     args_list <- base::formals(fun)
@@ -24,13 +25,21 @@ dcFunArgs <- function(fun, verbose=T)
         lft <- args_names[[i]]
         rgt <- paste(base::deparse(args_list[[i]]),collapse='')
         if(rgt!=''){
-            tmp <- paste(lft, '=', rgt, sep=' ')
-            eval(parse(text=tmp))
+            tmp <- paste(lft, '<-', rgt, sep=' ')
+            if(action==T){
+                base::eval(base::parse(text=tmp), envir=parent.frame())
+            }else{
+                base::eval(base::parse(text=tmp))
+            }
         }
     }
     
     if(verbose){
-        message(sprintf("In function '%s', %d arguments have been assigned with default values.", fun, length(args_names)), appendLF=T)
+        if(action==T){
+            message(sprintf("For the function '%s', %d arguments have been assigned with default values in the current environment.", fun, length(args_names)), appendLF=T)
+        }else{
+            message(sprintf("For the function '%s', there are %d arguments.", fun, length(args_names)), appendLF=T)
+        }
     }
     
     invisible(args_list)
